/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdbool.h>
#include <Elementary.h>
#include <app_debug.h>

#include "define.h"
#include "util/progressbar.h"
#include "util/util.h"

#define SLIDER_DEFAULT 0.05
#define SLIDER_STEP 0.03

#define PROGRESSBAR_INTERVAL 0.1

struct progressbar {
	Evas_Object *base;
	Evas_Object *slider;
	Ecore_Timer *timer;

	struct progressbar_ops *ops;
	void *ops_data;

	char *part_slider;
	char *part_total;
	char *part_progress;
	char *str_separator;

	enum progressbar_time_format fmt;
	int duration;
};

static void _update_progress_info(struct progressbar *m, int position)
{
	char progress[32] = {0,};

	util_time_string(progress, sizeof(progress), position, m->fmt);
	elm_object_part_text_set(m->base, m->part_progress, progress);
}

static void _update_time_info(struct progressbar *m, int position, int duration)
{
	char str[32] = {0,};
	char total[32] = {0,};

	util_time_string(str, sizeof(str), duration, m->fmt);
	snprintf(total, sizeof(total), "%s%s", m->str_separator, str);

	elm_object_part_text_set(m->base, m->part_total, total);

	_update_progress_info(m, position);
}

static Eina_Bool _timer_cb(void *data)
{
	struct progressbar *m;
	int position;

	if (!data)
		return ECORE_CALLBACK_CANCEL;

	m = data;

	position = m->ops->get_value(m->ops_data);

	elm_slider_value_set(m->slider, position);
	_update_progress_info(m, position);

	return ECORE_CALLBACK_RENEW;
}

void progressbar_start(struct progressbar *m)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	if (m->timer)
		ecore_timer_reset(m->timer);
	else
		m->timer = ecore_timer_add(PROGRESSBAR_INTERVAL, _timer_cb, m);
}

void progressbar_stop(struct progressbar *m)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	ecore_timer_del(m->timer);
	m->timer = NULL;

	elm_slider_value_set(m->slider, 0);
	_update_progress_info(m, 0);
}

void progressbar_pause(struct progressbar *m)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	if (m->timer)
		ecore_timer_freeze(m->timer);
}

void progressbar_resume(struct progressbar *m)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	if (m->timer)
		ecore_timer_thaw(m->timer);
}

void progressbar_show(struct progressbar *m)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	if (!m->part_slider || !m->slider) {
		_ERR("slider did not initialize");
		return;
	}

	evas_object_show(m->slider);
	elm_object_part_content_set(m->base, m->part_slider, m->slider);
}

void progressbar_hide(struct progressbar *m)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	if (!m->part_slider) {
		_ERR("slider did not initialize");
		return;
	}

	evas_object_hide(m->slider);
	elm_object_part_content_unset(m->base, m->part_slider);
}

void progressbar_reset(struct progressbar *m, int position, int duration)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	m->duration = duration;

	elm_slider_min_max_set(m->slider, 0, duration);
	elm_slider_step_set(m->slider, SLIDER_STEP);
	elm_slider_value_set(m->slider, position);

	_update_time_info(m, position, duration);
}

void progressbar_set_time_format(struct progressbar *m,
		enum progressbar_time_format fmt)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	m->fmt = fmt;
}

void progressbar_set_ops(struct progressbar *m,
		struct progressbar_ops *ops, void *data)
{
	if (!m || !ops) {
		_ERR("invalid parameter");
		return;
	}

	m->ops = ops;
	m->ops_data = data;
}

void progressbar_set_parts(struct progressbar *m, const char *slider,
		const char *total, const char *progress, const char *separator)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	if (slider)
		m->part_slider = strdup(slider);

	if (total)
		m->part_total = strdup(total);

	if (progress)
		m->part_progress = strdup(progress);

	if (separator)
		m->str_separator = strdup(separator);
}

struct progressbar *progressbar_create(Evas_Object *base, const char *style)
{
	struct progressbar *m;
	Evas_Object *obj;

	m = calloc(1, sizeof(*m));
	if (!m) {
		_ERR("failed to allocate");
		return NULL;
	}

	obj = elm_slider_add(base);
	if (!obj) {
		_ERR("failed to adding slider");
		free(m);
		return NULL;
	}

	if (style)
		elm_object_style_set(obj, style);

	elm_slider_indicator_show_set(obj, EINA_FALSE);
	elm_slider_indicator_show_on_focus_set(obj, EINA_FALSE);
	elm_slider_horizontal_set(obj, EINA_TRUE);
	elm_slider_step_set(obj, SLIDER_DEFAULT);

	/* FIXME: focus disabled */
	elm_object_focus_allow_set(obj, EINA_FALSE);

	m->base = base;
	m->slider = obj;
	m->fmt = PROG_TIME_FORMAT_FIT;

	return m;
}

void progressbar_destroy(struct progressbar *m)
{
	if (!m) {
		_ERR("invalid parameter");
		return;
	}

	progressbar_stop(m);

	free(m->part_slider);
	free(m->part_total);
	free(m->part_progress);
	free(m->str_separator);

	free(m);
}
